//=================================================================
//
//  Simple RF powermeter
//
//  (c) Dietmar Krause, DL2SBA
//
//  License see https://creativecommons.org/licenses/by/4.0/
//
#include <MsTimer2.h>

#include <Nextion.h>
#include <NextionPage.h>
#include <NextionButton.h>
#include <NextionText.h>

#include <PeakHold.h>

#include "data.h"
#include "config.h"

static NextionPage         pgSetup(nex,         2, 0,  "page2");
static NextionButton       b0_setup(nex,        2, 1,  "b0");

static NextionButton       bPeakHoldMinus(nex,  2, 4,  "b1");
static NextionButton       bPeakHoldPlus(nex,   2, 5,  "b2");
static NextionText         txtPeakHold(nex,     2, 6,  "t2");

static NextionButton       bPeakDecayMinus(nex, 2, 8,  "b3");
static NextionButton       bPeakDecayPlus(nex,  2, 10, "b4");
static NextionText         txtPeakDecay(nex,    2, 9,  "t4");

static NextionButton       bAverageMinus(nex,   2, 12, "b5");
static NextionButton       bAveragePlus(nex,    2, 13, "b6");
static NextionText         txtAverage(nex,      2, 14, "t6");

static NextionButton       bBacklightMinus(nex, 2, 16, "b7");
static NextionButton       bBacklightPlus(nex,  2, 18, "b8");
static NextionText         txtBacklight(nex,    2, 17, "t8");

static NextionButton       bPowerOffsetMinus(nex, 2, 20, "b9");
static NextionText         txtPowerOffset(nex,    2, 21, "t10");
static NextionButton       bPowerOffsetPlus(nex,  2, 22, "b10");

static NextionButton       bCalFreqMinus(nex, 2, 24, "b11");
static NextionText         txtCalFreq(nex,    2, 25, "t12");
static NextionButton       bCalFreqPlus(nex,  2, 26, "b12");

extern void switchToRawMode(NextionEventType type, INextionTouchable * widget);



//=================================================================
//
//  peak hold time
//
void cbPeakHoldMinus(NextionEventType type, INextionTouchable *widget) {
  if ( peakHoldCurrent > T_PEAKHOLD_MIN ) {
    peakHoldCurrent -= T_PEAKHOLD_STEP;
    itoa(peakHoldCurrent, dtostrfBuffer, 10);  txtPeakHold.setText(dtostrfBuffer);
  }
}

void cbPeakHoldPlus(NextionEventType type, INextionTouchable *widget) {
  if ( peakHoldCurrent < T_PEAKHOLD_MAX ) {
    peakHoldCurrent += T_PEAKHOLD_STEP;
    itoa(peakHoldCurrent, dtostrfBuffer, 10);  txtPeakHold.setText(dtostrfBuffer);
  }
}

//=================================================================
//
//  peak decay time
//
void cbPeakDecayMinus(NextionEventType type, INextionTouchable *widget) {
  if ( peakDecayCurrent > T_PEAKDECAY_MIN ) {
    peakDecayCurrent -= T_PEAKDECAY_STEP;
    itoa(peakDecayCurrent, dtostrfBuffer, 10);  txtPeakDecay.setText(dtostrfBuffer);
  }
}

void cbPeakDecayPlus(NextionEventType type, INextionTouchable *widget) {
  if ( peakDecayCurrent < T_PEAKDECAY_MAX ) {
    peakDecayCurrent += T_PEAKDECAY_STEP;
    itoa(peakDecayCurrent, dtostrfBuffer, 10);  txtPeakDecay.setText(dtostrfBuffer);
  }
}


//=================================================================
//
//  average
//
void cbAverageMinus(NextionEventType type, INextionTouchable *widget) {
  if ( currentBufferSize > DATA_BUFFER_SIZE_MIN ) {
    currentBufferSize -= DATA_BUFFER_SIZE_STEP;
    itoa(currentBufferSize, dtostrfBuffer, 10);     txtAverage.setText(dtostrfBuffer);
  }
}


void cbAveragePlus(NextionEventType type, INextionTouchable *widget) {
  if ( currentBufferSize < DATA_BUFFER_SIZE_MAX - 1 ) {
    currentBufferSize += DATA_BUFFER_SIZE_STEP;
    itoa(currentBufferSize, dtostrfBuffer, 10);     txtAverage.setText(dtostrfBuffer);
  }
}

//=================================================================
//
//  backlight
//
void cbBacklightMinus(NextionEventType type, INextionTouchable *widget) {
  if ( backlight > BACKLIGHT_MIN ) {
    backlight -= BACKLIGHT_STEP;
    itoa(backlight, dtostrfBuffer, 10);             txtBacklight.setText(dtostrfBuffer);
    nex.setBrightness(backlight, false);
  }
}


void cbBacklightPlus(NextionEventType type, INextionTouchable *widget) {
  if ( backlight < BACKLIGHT_MAX - 1 ) {
    backlight += BACKLIGHT_STEP;
    itoa(backlight, dtostrfBuffer, 10);             txtBacklight.setText(dtostrfBuffer);
    nex.setBrightness(backlight, false);
  }
}


//=================================================================
//
//  Poweroffset
//
void cbPowerOffsetMinus(NextionEventType type, INextionTouchable *widget) {
  if ( powerOffset_Index > 0 ) {
    --powerOffset_Index;
    itoa(powerSteps[powerOffset_Index], dtostrfBuffer, 10); txtPowerOffset.setText(dtostrfBuffer);
  }
}


void cbPowerOffsetPlus(NextionEventType type, INextionTouchable *widget) {
  if ( powerOffset_Index < sizeof(powerSteps) - 1 ) {
    ++powerOffset_Index;
    itoa(powerSteps[powerOffset_Index], dtostrfBuffer, 10); txtPowerOffset.setText(dtostrfBuffer);
  }
}

//=================================================================
//
//  Calibration Frequency
//
void cbCalFreqMinus(NextionEventType type, INextionTouchable *widget) {
  if ( calFreqIdx > 0 ) {
    --calFreqIdx;
    itoa(calFreqSteps[calFreqIdx], dtostrfBuffer, 10); txtCalFreq.setText(dtostrfBuffer);
  }
}


void cbCalFreqPlus(NextionEventType type, INextionTouchable *widget) {
  if ( calFreqIdx < CALFREQ_NUM - 1 ) {
    ++calFreqIdx;
    itoa(calFreqSteps[calFreqIdx], dtostrfBuffer, 10); txtCalFreq.setText(dtostrfBuffer);
  }
}


//=================================================================
//
//  do the stuff for peak measurement
//
void handleSetupMode( void ) {

}

//=================================================================
//
//  switch instrument into setup mode
//
void switchToSetupMode(NextionEventType type, INextionTouchable * widget) {
  // show page
  pgSetup.show();

  // show config data
  itoa(peakHoldCurrent, dtostrfBuffer, 10);               txtPeakHold.setText(dtostrfBuffer);
  itoa(peakDecayCurrent, dtostrfBuffer, 10);              txtPeakDecay.setText(dtostrfBuffer);
  itoa(currentBufferSize, dtostrfBuffer, 10);             txtAverage.setText(dtostrfBuffer);
  itoa(backlight, dtostrfBuffer, 10);                     txtBacklight.setText(dtostrfBuffer);
  itoa(powerSteps[powerOffset_Index], dtostrfBuffer, 10); txtPowerOffset.setText(dtostrfBuffer);
  itoa(calFreqSteps[calFreqIdx], dtostrfBuffer, 10);      txtCalFreq.setText(dtostrfBuffer);

  // stop background time
  MsTimer2::stop();

  // switch to mode
  mode = MODE_SETUP;
}



//=================================================================
//
//  save config and switch instrument into measurement mode
//
static void cbSwitchToRawMode(NextionEventType type, INextionTouchable * widget) {
  // write config data to eeprom
  writeConfig();

  // start timer again
  MsTimer2::start();

  // change mode
  switchToRawMode(type, widget);
}



//=================================================================
//
//
void initSetupMode( void ) {
  //
  b0_setup.attachCallback(&cbSwitchToRawMode);

  bPeakHoldMinus.attachCallback(&cbPeakHoldMinus);
  bPeakHoldPlus.attachCallback(&cbPeakHoldPlus);

  bPeakDecayMinus.attachCallback(&cbPeakDecayMinus);
  bPeakDecayPlus.attachCallback(&cbPeakDecayPlus);

  bAverageMinus.attachCallback(&cbAverageMinus);
  bAveragePlus.attachCallback(&cbAveragePlus);

  bBacklightMinus.attachCallback(&cbBacklightMinus);
  bBacklightPlus.attachCallback(&cbBacklightPlus);

  bPowerOffsetMinus.attachCallback(&cbPowerOffsetMinus);
  bPowerOffsetPlus.attachCallback(&cbPowerOffsetPlus);

  bCalFreqMinus.attachCallback(&cbCalFreqMinus);
  bCalFreqPlus.attachCallback(&cbCalFreqPlus);
}
